/++
 + Vorbis encode header module for D programming language
 +
 + Use, distribution and reproduction of this library source is 
 + governed by a BSD-STYLE souce license included with this source in 'COPYING'.
 + Please read these terms before distributing.
 +
 + Copyright(C) 2009-2011 Takuya Kurosawa
 +
 + Original code:
 +  vorbisenc.h (libvorbis-1.3.2)
 +  THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001
 +  by the Xiph.Org Foundation http://www.xiph.org/
 +/
module org.xiph.libvorbis.vorbisenc;

import org.xiph.libogg.types;
import org.xiph.libvorbis.codec;

extern (C)
{
	/++
	 + This is the primary function within libvorbisenc for setting up managed
	 + bitrate modes.
	 +
	 + Before this function is called, the \ref vorbis_info
	 + struct should be initialized by using vorbis_info_init() from the libvorbis
	 + API.  After encoding, vorbis_info_clear() should be called.
	 +
	 + The max_bitrate, nominal_bitrate, and min_bitrate settings are used to set
	 + constraints for the encoded file.  This function uses these settings to
	 + select the appropriate encoding mode and set it up.
	 +
	 + Params:
	 +  vi              = Pointer to an initialized \ref vorbis_info struct.
	 +  channels        = The number of channels to be encoded.
	 +  rate            = The sampling rate of the source audio.
	 +  max_bitrate     = Desired maximum bitrate (limit). -1 indicates unset.
	 +  nominal_bitrate = Desired average, or central, bitrate. -1 indicates unset.
	 +  min_bitrate     = Desired minimum bitrate. -1 indicates unset.
	 +
	 + Returns:
	 +  Zero for success, and negative values for failure.
	 +   0         = Success.
	 +   OV_EFAULT = Internal logic fault; indicates a bug or heap/stack corruption.
	 +   OV_EINVAL = Invalid setup request, eg, out of range argument.
	 +   OV_EIMPL  = Unimplemented mode; unable to comply with bitrate request.
	 +/
	int vorbis_encode_init(vorbis_info* vi, int channels, int rate, int max_bitrate, int nominal_bitrate, int min_bitrate);

	/++
	 + This function performs step-one of a three-step bitrate-managed encode
	 + setup.  It functions similarly to the one-step setup performed by \ref
	 + vorbis_encode_init but allows an application to make further encode setup
	 + tweaks using \ref vorbis_encode_ctl before finally calling \ref
	 + vorbis_encode_setup_init to complete the setup process.
	 +
	 + Before this function is called, the \ref vorbis_info struct should be
	 + initialized by using vorbis_info_init() from the libvorbis API.  After
	 + encoding, vorbis_info_clear() should be called.
	 +
	 + The max_bitrate, nominal_bitrate, and min_bitrate settings are used to set
	 + constraints for the encoded file.  This function uses these settings to
	 + select the appropriate encoding mode and set it up.
	 +
	 + Params:
	 +  vi              = Pointer to an initialized vorbis_info struct.
	 +  channels        = The number of channels to be encoded.
	 +  rate            = The sampling rate of the source audio.
	 +  max_bitrate     = Desired maximum bitrate (limit). -1 indicates unset.
	 +  nominal_bitrate = Desired average, or central, bitrate. -1 indicates unset.
	 +  min_bitrate     = Desired minimum bitrate. -1 indicates unset.
	 +
	 + Returns:
	 +  Zero for success, and negative for failure.
	 +   0         = Success
	 +   OV_EFAULT = Internal logic fault; indicates a bug or heap/stack corruption.
	 +   OV_EINVAL = Invalid setup request, eg, out of range argument.
	 +   OV_EIMPL  = Unimplemented mode; unable to comply with bitrate request.
	 +/
	int vorbis_encode_setup_managed(vorbis_info* vi, int channels, int rate, int max_bitrate, int nominal_bitrate, int min_bitrate);

	/++
	 + This is the primary function within libvorbisenc for setting up variable
	 + bitrate ("quality" based) modes.
	 +
	 +
	 + Before this function is called, the vorbis_info struct should be
	 + initialized by using vorbis_info_init() from the libvorbis API. After
	 + encoding, vorbis_info_clear() should be called.
	 +
	 + Params:
	 +  vi           = Pointer to an initialized vorbis_info struct.
	 +  channels     = The number of channels to be encoded.
	 +  rate         = The sampling rate of the source audio.
	 +  base_quality = Desired quality level, currently from -0.1 to 1.0 (lo to hi).
	 +
	 + Returns:
	 +  Zero for success, or a negative number for failure.
	 +   0         = Success
	 +   OV_EFAULT = Internal logic fault; indicates a bug or heap/stack corruption.
	 +   OV_EINVAL = Invalid setup request, eg, out of range argument.
	 +   OV_EIMPL  = Unimplemented mode; unable to comply with quality level request.
	 +/
	int vorbis_encode_setup_vbr(vorbis_info* vi, int channels, int rate, float quality);

	/++
	 + This is the primary function within libvorbisenc for setting up variable
	 + bitrate ("quality" based) modes.
	 +
	 +
	 + Before this function is called, the vorbis_info struct should be
	 + initialized by using vorbis_info_init() from the libvorbis API. After
	 + encoding, vorbis_info_clear() should be called.
	 +
	 + Params:
	 +  vi           = Pointer to an initialized vorbis_info struct.
	 +  channels     = The number of channels to be encoded.
	 +  rate         = The sampling rate of the source audio.
	 +  base_quality = Desired quality level, currently from -0.1 to 1.0 (lo to hi).
	 +
	 + Returns:
	 +  Zero for success, or a negative number for failure.
	 +   0         = Success
	 +   OV_EFAULT = Internal logic fault; indicates a bug or heap/stack corruption.
	 +   OV_EINVAL = Invalid setup request, eg, out of range argument.
	 +   OV_EIMPL  = Unimplemented mode; unable to comply with quality level request.
	 +/
	int vorbis_encode_init_vbr(vorbis_info* vi, int channels, int rate, float base_quality);

	/++
	 + This function performs the last stage of three-step encoding setup, as
	 + described in the API overview under managed bitrate modes.
	 +
	 + Before this function is called, the \ref vorbis_info struct should be
	 + initialized by using vorbis_info_init() from the libvorbis API, one of
	 + \ref vorbis_encode_setup_managed() or \ref vorbis_encode_setup_vbr() called to
	 + initialize the high-level encoding setup, and \ref vorbis_encode_ctl()
	 + called if necessary to make encoding setup changes.
	 + vorbis_encode_setup_init() finalizes the highlevel encoding structure into
	 + a complete encoding setup after which the application may make no further
	 + setup changes.
	 +
	 + After encoding, vorbis_info_clear() should be called.
	 +
	 + Params:
	 +  vi = Pointer to an initialized \ref vorbis_info struct.
	 +
	 + Returns:
	 +  Zero for success, and negative values for failure.
	 +   0         = Success.
	 +   OV_EFAULT = Internal logic fault; indicates a bug or heap/stack corruption.
	 +   OV_EINVAL = Attempt to use vorbis_encode_setup_init() without first
	 +               calling one of vorbis_encode_setup_managed() or vorbis_encode_setup_vbr() to
	 +               initialize the high-level encoding setup
	 +/
	int vorbis_encode_setup_init(vorbis_info* vi);

	/++
	 + This function implements a generic interface to miscellaneous encoder
	 + settings similar to the classic UNIX 'ioctl()' system call.  Applications
	 + may use vorbis_encode_ctl() to query or set bitrate management or quality
	 + mode details by using one of several \e request arguments detailed below.
	 + vorbis_encode_ctl() must be called after one of
	 + vorbis_encode_setup_managed() or vorbis_encode_setup_vbr().  When used
	 + to modify settings, \ref vorbis_encode_ctl() must be called before \ref
	 + vorbis_encode_setup_init().
	 +
	 + Params:
	 +  vi     = Pointer to an initialized vorbis_info struct.
	 +  number = Specifies the desired action; See \ref encctlcodes "the list of available requests".
	 +  arg    = void * pointing to a data structure matching the request argument.
	 +
	 + Returns:
	 +  0         = Success. Any further return information (such as the result of a
	 +              query) is placed into the storage pointed to by *arg.
	 +  OV_EINVAL = Invalid argument, or an attempt to modify a setting after
	 +              calling vorbis_encode_setup_init().
	 +  OV_EIMPL  = Unimplemented or unknown request
	 +/
	int vorbis_encode_ctl(vorbis_info* vi, int number, void* arg);
}

/++
 + The ovectl_ratemanage2_arg structure is used with vorbis_encode_ctl() and
 + the OV_ECTL_RATEMANAGE2_GET and OV_ECTL_RATEMANAGE2_SET calls in order to
 + query and modify specifics of the encoder's bitrate management
 + configuration.
 +/
struct ovectl_ratemanage2_arg
{
	/// nonzero if bitrate management is active
	int management_active;

	/// Lower allowed bitrate limit in kilobits per second
	int bitrate_limit_min_kbps;

	/// Upper allowed bitrate limit in kilobits per second
	int bitrate_limit_max_kbps;

	/// Size of the bitrate reservoir in bits
	int bitrate_limit_reservoir_bits;

	/// Regulates the bitrate reservoir's preferred fill level in a range from 0.0 to 1.0;
	/// 0.0 tries to bank bits to buffer against future bitrate spikes, 1.0
 	/// buffers against future sudden drops in instantaneous bitrate.
	/// Default is 0.1
	double bitrate_limit_reservoir_bias;

	/// Average bitrate setting in kilobits per second
	int bitrate_average_kbps;

	/// Slew rate limit setting for average bitrate adjustment; sets the minimum
	/// time in seconds the bitrate tracker may swing from one extreme to the
	/// other when boosting or damping average bitrate.
	double bitrate_average_damping;
}

/+
 + vorbis_encode_ctl() codes
 +
 + These values are passed as the \c number parameter of vorbis_encode_ctl().
 + The type of the referent of that function's \c arg pointer depends on these codes.
 +/

/++
 + Query the current encoder bitrate management setting.
 +
 + vorbis_encode_ctl.arg = ovectl_ratemanage2_arg*
 +
 + Used to query the current encoder bitrate management setting. Also used to
 + initialize fields of an ovectl_ratemanage2_arg structure for use with
 +/
immutable uint OV_ECTL_RATEMANAGE2_GET = 0x14;

/++
 + Set the current encoder bitrate management settings.
 +
 + vorbis_encode_ctl.arg = ovectl_ratemanage2_arg*
 +
 + Used to set the current encoder bitrate management settings to the values
 + listed in the ovectl_ratemanage2_arg. Passing a NULL pointer will disable
 + bitrate management.
 +/
immutable uint OV_ECTL_RATEMANAGE2_SET = 0x15;

/++
 + Returns the current encoder hard-lowpass setting (kHz) in the double
 + pointed to by arg.
 +
 + vorbis_encode_ctl.arg = double*
 +/
immutable uint OV_ECTL_LOWPASS_GET = 0x20;

/++
 + Sets the encoder hard-lowpass to the value (kHz) pointed to by arg. Valid
 + lowpass settings range from 2 to 99.
 +
 + vorbis_encode_ctl.arg = double*
 +/
immutable uint OV_ECTL_LOWPASS_SET = 0x21;

/++
 +  Returns the current encoder impulse block setting in the double pointed
 +  to by arg.
 +
 + vorbis_encode_ctl.arg = double*
 +/
immutable uint OV_ECTL_IBLOCK_GET = 0x30;

/++
 + Sets the impulse block bias to the the value pointed to by arg.
 +
 + vorbis_encode_ctl.arg = double*
 +
 + Valid range is -15.0 to 0.0 [default]. A negative impulse block bias will
 + direct to encoder to use more bits when incoding short blocks that contain
 + strong impulses, thus improving the accuracy of impulse encoding.
 +/
immutable uint OV_ECTL_IBLOCK_SET = 0x31;

/++
 + Returns the current encoder coupling setting in the int pointed to by arg.
 +
 + vorbis_encode_ctl.arg = int*
 +/
immutable uint OV_ECTL_COUPLING_GET = 0x40;

/++
 + Enables/disables channel coupling in multichannel encoding according to arg.
 +
 + vorbis_encode_ctl.arg = int*
 +
 + Zero disables channel coupling for multichannel inputs, nonzer enables
 + channel coupling.  Setting has no effect on monophonic encoding or
 + multichannel counts that do not offer coupling.  At present, coupling is
 + available for stereo and 5.1 encoding.
 +/
immutable uint OV_ECTL_COUPLING_SET = 0x41;

// deprecated rate management supported only for compatability
deprecated
{
	immutable uint OV_ECTL_RATEMANAGE_GET  = 0x10;
	immutable uint OV_ECTL_RATEMANAGE_SET  = 0x11;
	immutable uint OV_ECTL_RATEMANAGE_AVG  = 0x12;
	immutable uint OV_ECTL_RATEMANAGE_HARD = 0x13;

	struct ovectl_ratemanage_arg
	{
		int management_active;

		int bitrate_hard_min;
		int bitrate_hard_max;
		double bitrate_hard_window;

		int bitrate_av_lo;
		int bitrate_av_hi;
		double bitrate_av_window;
		double bitrate_av_window_center;
	}
}
